-- Written by Skunkcrafts for Furax84
-- Feel to free to copy any part of the code. Please only modify copies

-- main.lua
airports	= {}
portails_var	= {}
anims		= {linear = {}}
anims_portail		= {linear = {}}

sasl.options.setInteractivity(false)
sasl.options.setAircraftPanelRendering(false)
sasl.options.set3DRendering(false)

local xp_path	= sasl.getXPlanePath()

local acf_lat	= globalPropertyd("sim/flightmodel/position/latitude")
local acf_lon	= globalPropertyd("sim/flightmodel/position/longitude")
local ftime		= globalPropertyf("sim/operation/misc/frame_rate_period")
local period	= 0

local function anim_linear(icao, idx, target)
	local id = "icao" .. idx
	if airports[icao].linear.timings[idx+1] <= 0 then airports[icao].linear.timings[idx+1] = 1 end
	anims.linear[id] = {icao = icao, idx = idx+1, target = target, speed = 1 / airports[icao].linear.timings[idx+1]}
end

local function anim_portails(icao, idx, target)
	local id = "icao" .. idx
	if portails_var[icao].linear.timings[idx+1] <= 0 then portails_var[icao].linear.timings[idx+1] = 1 end
	anims_portail.linear[id] = {icao = icao, idx = idx+1, target = target, speed = 1 / portails_var[icao].linear.timings[idx+1]}
end

local function do_linears()
	for id, anim in pairs(anims.linear) do
		local value = get(airports[anim.icao].linear.array, anim.idx)
		local delta = anim.speed * period
		if value > anim.target then
			value = math.max(anim.target, value - delta)
		else
			value = math.min(anim.target, value + delta)
		end
		set(airports[anim.icao].linear.array, value, anim.idx)
		if value == anim.target then
			anims.linear[id] = nil
		end
	end
end

local function do_linears_portails()
	for id, anim in pairs(anims_portail.linear) do
		local value = get(portails_var[anim.icao].linear.array, anim.idx)
		local delta = anim.speed * period
		if value > anim.target then
			value = math.max(anim.target, value - delta)
		else
			value = math.min(anim.target, value + delta)
		end
		set(portails_var[anim.icao].linear.array, value, anim.idx)
		if value == anim.target then
			anims_portail.linear[id] = nil
		end
	end
end

function string:unpack_spc()
	local r = {}
	local n = 0
	for w in self:gmatch("%S+") do n = n + 1 r[n] = w end
	return unpack(r)
end

local function geoDistanceRadNM(radLat1, radLon1, radLat2, radLon2)
	local deltaLat = radLat2-radLat1
	local deltaLon = radLon2-radLon1

	local sLat = math.sin(deltaLat/2)
	local sLon = math.sin(deltaLon/2)
	local a = sLat*sLat + math.cos(radLat1) * math.cos(radLat2) * sLon*sLon
	local c = 2 * math.atan2(math.sqrt(a), math.sqrt(1-a))
	return 3437.75 * c
end

local function set_airport(icao)
	airports[icao] = {
		linear	= {
			timings = {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1},
			array	= createGlobalPropertyfa("Furax84/Scenery/" .. icao .. "/linear", {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0})
		}
	}
	logInfo("Found " .. icao)
	return icao
end

--- perso

local function set_portail(icao)
	portails_var[icao] = {
		linear	= {
			timings = {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1},
			array	= createGlobalPropertyfa("Furax84/Portail/" .. icao .. "/linear", {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0})
		}
	}
	logInfo("Found " .. icao)
	return icao
end

local function check_scenery(fname)
	if not isFileExists(fname) then
		return false
	end

	local file, error = io.open(fname, "r")
	if nil == file then
		logInfo("Error reading " .. fname .. " : " .. error .. ". Cannot parse Furax84 file")
		return false
	end

	local processing = false
	local linears	= 0
	for line in file:lines() do
		line = line:match("[^\r\n]+") or ""
		local id, value = line:unpack_spc()
		if id and value then
			if id == "AIRPORT" then
				processing = set_airport(value)
			end
			if processing and id == "LINEAR" and tonumber(value) then
				linears = linears + 1
				airports[processing].linear.timings[linears] = tonumber(value)
			end
		end
	end
	file:close()
end

local function check_scenery_portail(fname)
	if not isFileExists(fname) then
		return false
	end

	local file, error = io.open(fname, "r")
	if nil == file then
		logInfo("Error reading " .. fname .. " : " .. error .. ". Cannot parse Furax84 file")
		return false
	end

	local processing = false
	local linears	= 0
	for line in file:lines() do
		line = line:match("[^\r\n]+") or ""
		local id, value = line:unpack_spc()
		if id and value then
			if id == "PORTAIL" then
				processing = set_portail(value)
			end
			if processing and id == "LINEAR" and tonumber(value) then
				linears = linears + 1
				portails_var[processing].linear.timings[linears] = tonumber(value)
			end
		end
	end
	file:close()
end

-- do not use
-- will miss newly installed airports if ran at load time
-- scenery will fail to match dataref if ran after scenery is loaded
local function parse_scenery_ini()
	local fname	= xp_path .. "Custom Scenery/scenery_packs.ini"
	if not isFileExists(fname) then
		logInfo("Scenery file " .. ini .. " not found, cannot parse sceneries")
		return false
	end

	local file, error = io.open(fname, "r")
	if nil == file then
		logInfo("Error reading " .. ini .. " : " .. error .. ". Cannot parse sceneries")
		return false
	end

	for line in file:lines() do
		line = line:match("[^\r\n]+") or ""
		local id, value = line:unpack_spc()
		if id and value then
			if id == "SCENERY_PACK"  and value ~= "*GLOBAL_AIRPORTS*" then
				check_scenery(line:sub(14))
			end
		end
	end
	file:close()
end

-- TODO: check if symlinks are followed
local function parse_scenery_folder()
	local scenery_path = xp_path .. "Custom Scenery/"
	logInfo("Parsing", scenery_path)
	local content = sasl.listFiles(scenery_path)
	for _, item in ipairs(content) do
		if item.type == "directory" then
			check_scenery(scenery_path .. item.name .. "/furax84.txt")
		end
	end
end

-- TODO: check if symlinks are followed portails
local function parse_portail_folder()
	local scenery_path = xp_path .. "Custom Scenery/"
	logInfo("Parsing", scenery_path)
	local content = sasl.listFiles(scenery_path)
	for _, item in ipairs(content) do
		if item.type == "directory" then
			check_scenery_portail(scenery_path .. item.name .. "/portails.txt")
		end
	end
end

local function get_closest_airport()
	local lat_rad = math.rad(get(acf_lat))
	local lon_rad = math.rad(get(acf_lon))
	local min_dist = 999999
	local closest
	for icao, airport in pairs(airports) do
		local dist = geoDistanceRadNM(lat_rad, lon_rad, airport.lat, airport.lon)
		if dist < min_dist then
			closest		= icao
			min_dist	= dist
		end
	end
	return closest
end

local function get_closest_portail()
	local lat_rad = math.rad(get(acf_lat))
	local lon_rad = math.rad(get(acf_lon))
	local min_dist = 999999
	local closest
	for icao, airport in pairs(airports) do
		local dist = geoDistanceRadNM(lat_rad, lon_rad, airport.lat, airport.lon)
		if dist < min_dist then
			closest		= icao
			min_dist	= dist
		end
	end
	return closest
end

local function linear_open(idx)  local icao = get_closest_airport() if not icao then logInfo("No Furax84 scenery") end anim_linear(icao, idx, 1) end
local function linear_close(idx) local icao = get_closest_airport() if not icao then logInfo("No Furax84 scenery") end anim_linear(icao, idx, 0)  end

local function make_cmd_linear_open(idx)  return function(p) if p > 0 then return end linear_open(idx)  end end
local function make_cmd_linear_close(idx) return function(p) if p > 0 then return end linear_close(idx) end end

-- perso Portail
local function portail_open(idx)  local icao = get_closest_portail() if not icao then logInfo("No Furax84 scenery") end anim_portails(icao, idx, 1) end
local function portail_close(idx) local icao = get_closest_portail() if not icao then logInfo("No Furax84 scenery") end anim_portails(icao, idx, 0)  end

local function make_cmd_portail_open(idx)  return function(p) if p > 0 then return end portail_open(idx)  end end
local function make_cmd_portail_close(idx) return function(p) if p > 0 then return end portail_close(idx) end end


for i = 0, 19 do
	sasl.registerCommandHandler(createCommand("VFR_Scenery_animate/Hangar/open_" .. i , "Hangar open #" .. i) , 1, make_cmd_linear_open(i))
	sasl.registerCommandHandler(createCommand("VFR_Scenery_animate/Hangar/close_" .. i, "Hangar close #" .. i), 1, make_cmd_linear_close(i))
	sasl.registerCommandHandler(createCommand("VFR_Scenery_animate/Portail/open_" .. i , "Portail open #" .. i) , 1, make_cmd_portail_open(i))
	sasl.registerCommandHandler(createCommand("VFR_Scenery_animate/Portail/close_" .. i, "Portail close #" .. i), 1, make_cmd_portail_close(i))
end

local menu = {}
menu.master	= sasl.appendMenuItem (PLUGINS_MENU_ID, "VFR Scenery animate") -- Ajout du nom du menu au menu principal plugin
menu.main	= sasl.createMenu("", PLUGINS_MENU_ID, menu.master) -- creation du menu VFR Scenery animate
menu.linear		= sasl.appendMenuItem(menu.main, "Linear") -- Ajout du nom de l'ICAO de l'icao a menu.main
menu.linear_id	= sasl.createMenu("", menu.main, menu.linear) -- Creation du menu

local hangars	= sasl.appendMenuItem(menu.linear_id, "Hangar")
local hangars_id	= sasl.createMenu("", menu.linear_id, hangars)

local portails	= sasl.appendMenuItem(menu.linear_id, "Portails")
local portails_id	= sasl.createMenu("", menu.linear_id, portails)

for i = 0, 19 do
		local menu_3	= sasl.appendMenuItem(hangars_id, "Hangar #" .. i)
		local menu_4	= sasl.createMenu("", hangars_id, menu_3)
		sasl.appendMenuItem(menu_4, "Open ", function() linear_open (i) end)
		sasl.appendMenuItem(menu_4, "Close", function() linear_close(i) end)
end

for i = 0, 19 do
		local menu_5	= sasl.appendMenuItem(portails_id, "Portail #" .. i)
		local menu_6	= sasl.createMenu("", portails_id, menu_5)
		sasl.appendMenuItem(menu_6, "Open ", function() portail_open (i) end)
		sasl.appendMenuItem(menu_6, "Close", function() portail_close(i) end)
end

function onModuleInit()
	for icao, airport in pairs(airports) do
		local id = sasl.findNavAid(nil, icao , nil, nil, nil, NAV_AIRPORT)
		if id == NAV_NOT_FOUND then
			logInfo("Airport " .. icao .. " not found")
			airport.lat = 0
			airport.lon = 0
		else
			local _ , lat, lon = sasl.getNavAidInfo(id)
			airport.lat = math.rad(lat)
			airport.lon = math.rad(lon)
		end
	end
end

function update()
	-- cosmetic but does not takes much resources
	sasl.setMenuItemName(menu.main, menu.linear, (get_closest_airport() or "----"))
	period = get(ftime)
	do_linears()
	do_linears_portails()
end

parse_scenery_folder()
parse_portail_folder()